@@ -4,11 +4,14 @@
 
 setMethod("crossTable", signature(x = "itemMatrix"),
     function(x, ...) {
+## todo: test
+## m <- .Call("R_crosstab_ngCMatrix", x, NULL, TRUE)
         m <- as(tcrossprod(as(x, "dgCMatrix")), "matrix")
 
         ## FIXME:
         ## dsCMatrix -> matrix in Matrix somehow does not handle dimnames!
-        dimnames(m) <- list(itemLabels(x), itemLabels(x))
+        if (is.null(dimnames))
+            dimnames(m) <- list(itemLabels(x), itemLabels(x))
 
         m
     })
