@@ -11,7 +11,7 @@ setMethod("length", signature(x = "rules"),
     )
 
 setMethod("size", signature(x = "rules"),
-    function(x) size(x@lhs) + size(x@rhs)
+    function(x, ...) size(x@lhs) + size(x@rhs)
     )
 
 
@@ -92,37 +92,54 @@ setMethod("generatingItemsets", signature(x = "rules"),
 ## subset, combine
 
 setMethod("[", signature(x = "rules", i = "ANY", j = "ANY", drop = "ANY"),
-        function(x, i, j, ..., drop)            #]
-    {
-        if (!missing(j)) stop("incorrect number of dimensions (j not possible)")
+    function(x, i, j, ..., drop) {           #]
+
+        if (!missing(j)) 
+            stop("incorrect dimension (j not possible)")
         if (missing(i)) return(x)
-        y <- x
         slots <- intersect(slotNames(x), c("lhs", "rhs"))
-        for (sl in slots) slot(y, sl) <- slot(x, sl)[i]
-        y@quality <- x@quality[i,,drop=FALSE]
-        return(y)
+        for (s in slots) 
+            slot(x, s) <- slot(x, s)[i]
+        x@quality <- x@quality[i,, drop = FALSE]
+        x
     })
 
+# fixme: the code above suggests it is possible that rhs
+#        and/or lhs need not be present. however, the
+#        code below does.
+
 
 setMethod("c", signature(x = "rules"),
     function(x, ..., recursive = FALSE) {
-        args <- list(...)  
-        if(length(args) == 0) return(x)
+#        args <- list(...)  
+#        if(length(args) == 0) return(x)
+#
+#        # build quality data.frame first
+#        # uses .combineQuality() defined in itemsets.R
+#        q <- x@quality
+#        for(i in 1:length(args)) {
+#            q <- .combineQuality(q, args[[i]]@quality)
+#        }
+#
+#
+#        # create joint itemMatrix
+#        lhs <- lapply(list(...), FUN = function(i) i@lhs)
+#        rhs <- lapply(list(...), FUN = function(i) i@rhs)
+#        new("rules", lhs = c(x@lhs, lhs, recursive = TRUE),
+#            rhs = c(x@rhs, rhs, recursive = TRUE),
+#            quality = q)
 
-        # build quality data.frame first
-        # uses .combineQuality() defined in itemsets.R
-        q <- x@quality
-        for(i in 1:length(args)) {
-            q <- .combineQuality(q, args[[i]]@quality)
+        args <- list(...)
+        if (recursive)
+            args <- unlist(args)
+        for (y in args) {
+            if (!inherits(y, "rules"))
+                stop("can combine rules only")
+            new("rules", lhs     = c(x@lhs, y@lhs), 
+                         rhs     = c(x@rhs, y@rhs),
+                         quality = .combineMeta(x, y, "quality"))
         }
-
-
-        # create joint itemMatrix
-        lhs <- lapply(list(...), FUN = function(i) i@lhs)
-        rhs <- lapply(list(...), FUN = function(i) i@rhs)
-        new("rules", lhs = c(x@lhs, lhs, recursive = TRUE),
-            rhs = c(x@rhs, rhs, recursive = TRUE),
-            quality = q)
+        x
     })
 
 
@@ -134,9 +151,9 @@ setMethod("c", signature(x = "rules"),
     tmp <- LIST(x@lhs, decode = FALSE)
     rhs <- LIST(x@rhs, decode = FALSE)
 
-    for (i in 1:length(x)) 
     ## -> is used to distinguish between lhs and rhs
-    tmp[[i]] <- c(tmp[[i]], "->" , rhs[[i]])
+    for (i in 1:length(x)) 
+        tmp[[i]] <- c(tmp[[i]], "->" , rhs[[i]])
     tmp
 }
 
@@ -151,7 +168,7 @@ setMethod("match", signature(x = "rules", table = "rules"),
     function(x,  table, nomatch = NA, incomparables = FALSE) {
         match(.joinedList(x), .joinedList(table),
             nomatch = nomatch, incomparables = incomparables)
-    })
+})
 
 
 ##************************************************
@@ -160,10 +177,12 @@ setMethod("match", signature(x = "rules", table = "rules"),
 
 setMethod("summary", signature(object = "rules"), 
     function(object, ...) {
+        sizes <- size(object@lhs) + size(object@rhs)
+        
         new("summary.rules", 
             length = length(object),
-            lengths = table(size(object@lhs)+size(object@rhs)),
-            lengthSummary = summary(size(object@lhs)+size(object@rhs)),
+            lengths = table(sizes),
+            lengthSummary = summary(sizes),
             quality = if(length(object@quality) > 0) summary(object@quality)
             else summary(NULL))
     })
